import json
from django.conf import settings
from django import forms
from django.shortcuts import render
from django.utils.safestring import mark_safe
from default import SUMMERNOTE_DEFAULT_CONFIG


class SummernoteWidget(forms.Textarea):

    def _media(self):
        django_jquery_adapter = settings.STATIC_URL + 'js/django-jquery-adapter.js'
        js = (django_jquery_adapter,) + self.assets.get('js')
        if self.load_init:
            js+= (settings.STATIC_URL + 'js/summernote-init.js',)
        return forms.Media(css=self.assets.get('css'),
                           js=js)
    media = property(_media)


    def get_editor(self):
        return self.editor_conf


    def __init__(self, editor_conf=None, load_init=True, async=None, *args, **kwargs):

        self.load_init = load_init
        self.async = async
        self.general_config = getattr(settings, 'SUMMERNOTE_CONFIG', SUMMERNOTE_DEFAULT_CONFIG)
        self.editors = self.general_config.get('editors')
        self.editor_conf=self.editors.get(editor_conf or 'default', self.editors.get('default'))


        self.assets = self.general_config.get('assets', SUMMERNOTE_DEFAULT_CONFIG.get('assets'))


        super(SummernoteWidget, self).__init__(*args, **kwargs)


    def render(self, name, value, attrs=None):
        return mark_safe(render(None, 'ideia_summernote/default.html', { 'self': self,
                                                                        'name': name,
                                                                         'value': value,
                                                                        'async': self.async,
                                                                        'config': json.dumps(self.editor_conf),
                                                                         'attrs': attrs

                                                                        }).content)


_2_MB = 2048000

SUMMERNOTE_DEFAULT_CONFIG = {

    'restrict_to_user': False,
    'use_path_user': False,
    'upload_only_image_files': False,
    'maximum_image_upload': _2_MB,

    'assets': {
        'js': (
            'https://code.jquery.com/jquery-2.2.4.min.js',
            'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/summernote/0.8.1/summernote.min.js', ),

        'css': {
            'all': (
                'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css',
                'https://cdnjs.cloudflare.com/ajax/libs/summernote/0.8.1/summernote.css',
            )
        }
    },

    'editors': {
        'default': {
            'airMode': False,
            'toolbar': [
                ['style', ['bold', 'italic', 'underline', 'clear']],
                ['font', ['strikethrough', 'superscript', 'subscript']],
                ['fontsize', ['fontsize']],
                ['color', ['color']],
                ['para', ['ul', 'ol', 'paragraph']],
                ['height', ['height']]
            ],
            'popover': {
                'air':[
                    ['para', ['ul', 'ol', 'paragraph']],
                ]
            }
        },

    }
}


import os

from django.http import JsonResponse
from django.template.defaultfilters import slugify
from django.views.generic import View
from django.core.files.storage import default_storage
from django.conf import settings
from django.utils.translation import ugettext as _

from default import SUMMERNOTE_DEFAULT_CONFIG
from ..ideia_summernote.local_exceptions import FileSizeNotSupported

SUMMERNOTE_SETTINGS =  getattr(settings, 'SUMMERNOTE_CONFIG', SUMMERNOTE_DEFAULT_CONFIG)
MAXIMUM_SIZE = getattr(SUMMERNOTE_SETTINGS, 'maximum_image_upload', SUMMERNOTE_DEFAULT_CONFIG['maximum_image_upload'])


def check_sizes(files):

    if files:
        for file in files.values():
            if file.size > MAXIMUM_SIZE:
                raise FileSizeNotSupported()


class Upload(View):

    def post(self, request):

        files = request.FILES
        urls = []

        is_user_authenticated = request.user.is_authenticated()
        if SUMMERNOTE_SETTINGS['restrict_to_user']:
            if not is_user_authenticated:
                return JsonResponse(data={'message': _('Only authenticated users can submit images!')}, status=403)


        if request.FILES:

            try:
                check_sizes(request.FILES)
            except FileSizeNotSupported:
                return JsonResponse(data={'message': _('File size not supported!')}, status=400)

            for file in files.values():
                user_path=''

                if SUMMERNOTE_SETTINGS['use_path_user'] and is_user_authenticated:
                    user_path = request.user.username

                filename = file.name
                ext = slugify(filename.split('.')[-1])
                name = slugify(".".join(filename.split('.')[0:-1]))
                filename = "{0}.{1}".format(name, ext)

                path = os.path.join(settings.MEDIA_ROOT, user_path, filename)
                path = default_storage.save(path, file)
                if path:
                    urls.append(os.path.join(settings.MEDIA_URL, user_path, filename))

        return JsonResponse(data={'urls': urls})


from django.conf import settings
import re
from django.forms.fields import CharField
from django.db import models
from ideia_summernote.widget import SummernoteWidget
import bleach


class SummernoteFormField(CharField):

    def __init__(self, editor_conf='default', load_init=True, async=False, *args, **kwargs):

        kwargs.update({'widget': SummernoteWidget(editor_conf=editor_conf, load_init=load_init, async=async)})
        super(SummernoteFormField, self).__init__(*args, **kwargs)

    def default_tag_map(self):

        default_attrs = ['style', 'line-height']

        map = {
            'bold': {
                'b': default_attrs,
            },
            'italic': {
                'i': default_attrs
            },
            'underline': {
                'u': default_attrs
            },
            'superscript':{
                'sub': default_attrs
            },
            'subscript': {
                'sup': default_attrs
            },
            'strikethrough': {
                'strike': default_attrs
            },
            'fontsize': {
                'span': default_attrs
            },
            'ul': {
                'ul': default_attrs,
                'li': default_attrs
            },
            'ol': {
                'ol': default_attrs,
                'li': default_attrs
            },
            'picture': {
                'img': default_attrs + ['src']
            }
        }

        return map

    def clean(self, value):
        value = super(SummernoteFormField, self).clean(value)
        try:
            enabled_tags = self.get_editor().get('enabledTags', {})
            toolbar_conf = self.get_editor().get('toolbar', [])
            enabled_styles = self.get_editor().get('enabledStyles', {})
            default_map = self.default_tag_map()

            enabled_keys = []

            for tconf_items in toolbar_conf:
                t_item = tconf_items[1] if len(tconf_items) > 0 else None
                if not t_item:
                    continue
                [enabled_keys.append(i) for i in t_item]

            enabled_keys = list(set(enabled_keys))

            attrs = {}
            tags = []

            for key, item in default_map.items():
                if key not in enabled_keys:
                    continue

                [tags.append(i) for i in item.keys()]
                attrs.update(item)

            tags = list(set(tags))

            for etag, eattrs in enabled_tags.items():
                tags.append(etag)
                attrs.update({'%s' % etag: eattrs})

            value = bleach.clean(value, tags=tags, attributes=attrs, styles=enabled_styles, strip=True)
        except Exception, e:
            print e.message

        return value

    def get_editor(self):
        return self.widget.get_editor()


class SummernoteField(models.TextField):

    def __init__(self, editor_conf='default', load_init=True, async=False, *args, **kwargs):
        self.editor_conf = editor_conf
        self.load_init=load_init
        self.async=async
        super(SummernoteField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):

        defaults = {
            'form_class':  SummernoteFormField,
            'editor_conf': self.editor_conf,
            'load_init': self.load_init,
            'async': self.async
        }

        defaults.update(kwargs)
        return super(SummernoteField, self).formfield(**defaults)